//////////////////////////////////////////////////////////////////////////////
//
//  opu_tools_fix.cpp
//
//  Description:
//      Contains Unigraphics entry points for the application.
//
//////////////////////////////////////////////////////////////////////////////

#define _CRT_SECURE_NO_DEPRECATE 1

//  Include files

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <uf.h>
#include <uf_defs.h>
#include <uf_exit.h>
#include <uf_ui.h>
#include <uf_styler.h>
#include <uf_mb.h>

#include <uf_attr.h>
#include <uf_obj.h>
#include <uf_param.h>
#include <uf_param_indices.h>
#include <uf_ui_ont.h>
#include <uf_cam.h>
#include <uf_setup.h>
#include <uf_ncgroup.h>
#include <uf_oper.h>
#include <uf_ugopenint.h>
#include <uf_part.h>
#include <uf_assem.h>
#include <uf_ugmgr.h>

/*
#if ! defined ( __hp9000s800 ) && ! defined ( __sgi ) && ! defined ( __sun )
#	include <strstream>
	using std::ostrstream;
	using std::endl;
	using std::ends;
#else
#	include <strstream.h>
#endif
#include <iostream.h>
*/
#include "opu_tools_fix.h"



/* The following definition defines the number of callback entries */
/* in the callback structure:                                      */
/* UF_STYLER_callback_info_t MT_cbs */
#define MT_CB_COUNT ( 9 + 1 ) /* Add 1 for the terminator */

/*--------------------------------------------------------------------------
The following structure defines the callback entries used by the
styler file.  This structure MUST be passed into the user function,
UF_STYLER_create_dialog along with MT_CB_COUNT.
--------------------------------------------------------------------------*/
static UF_STYLER_callback_info_t MT_cbs[MT_CB_COUNT] =
{
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CONSTRUCTOR_CB  , 0, MT_construct_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_APPLY_CB        , 0, MT_apply_cb},
 {MT_TOOL_TOOLS         , UF_STYLER_ACTIVATE_CB     , 0, MT_toolpalette_cb},
 {MT_LIST_TOOLS         , UF_STYLER_ACTIVATE_CB     , 0, MT_list_act_cb},
 {MT_LIST_TOOLS         , UF_STYLER_DOUBLE_CLICK_CB , 1, MT_list_dbl_cb},
 {MT_INT_TOOLS          , UF_STYLER_ACTIVATE_CB     , 0, MT_int_tools_cb},
 {MT_ACTION_LIST        , UF_STYLER_ACTIVATE_CB     , 0, MT_action_list_act_cb},
 {MT_TOGGLE_TOOLS       , UF_STYLER_VALUE_CHANGED_CB, 0, MT_tog_tools_cb},
 {MT_BLO_TOOLS          , UF_STYLER_ACTIVATE_CB     , 0, MT_blo_tools_act_cb},
 {UF_STYLER_NULL_OBJECT, UF_STYLER_NO_CB, 0, 0 }
};



/*--------------------------------------------------------------------------
UF_MB_styler_actions_t contains 4 fields.  These are defined as follows:

Field 1 : the name of your dialog that you wish to display.
Field 2 : any client data you wish to pass to your callbacks.
Field 3 : your callback structure.
Field 4 : flag to inform menubar of your dialog location.  This flag MUST
          match the resource set in your dialog!  Do NOT ASSUME that changing
          this field will update the location of your dialog.  Please use the
          UIStyler to indicate the position of your dialog.
--------------------------------------------------------------------------*/
static UF_MB_styler_actions_t actions[] = {
    { "opu_tools_fix.dlg",  NULL,   MT_cbs,  UF_MB_STYLER_IS_NOT_TOP },
    { NULL,  NULL,  NULL,  0 } /* This is a NULL terminated list */
};



/****************************************************************************/

static void PrintErrorMessage( int errorCode );

/****************************************************************************/

#define UF_CALL(X) (report( __FILE__, __LINE__, #X, (X)))

static int report( char *file, int line, char *call, int irc)
{
  if (irc)
  {
     char    messg[133];
     printf("%s, line %d:  %s\n", file, line, call);
     (UF_get_fail_message(irc, messg)) ?
       printf("    returned a %d\n", irc) :
       printf("    returned error %d:  %s\n", irc, messg);
  }
  return(irc);
}

int generat = 0;
int num_tool = 0 ;
static logical cycleGeneratePrg( tag_t   tag, void   *data );

/* */
/*----------------------------------------------------------------------------*/
static logical cycleGeneratePrg( tag_t   tag, void   *data )
{
   char     str[ 132 + 1 ] ;
   char     name [ MAX_ENTITY_NAME_SIZE+1 ] ;
   int      num ;
   logical  is_ok, gened;
   //logical  is_group ;
   //UF_NCGROUP_is_group( tag, &is_group );
   //if( is_group == TRUE ) return( TRUE );

   UF_CALL( UF_PARAM_ask_int_value(tag,UF_PARAM_TL_NUMBER,&num) );
   if (num_tool!=num) {

     UF_CALL( UF_PARAM_set_int_value(tag,UF_PARAM_TL_NUMBER,-99) );
     UF_CALL( UF_PARAM_set_int_value(tag,UF_PARAM_TL_NUMBER,num_tool) );

     if (generat) {
       /* Validate the parameters of this Operation. */
       UF_CALL( UF_PARAM_check( tag, &is_ok ) );
       if( is_ok == FALSE ) { return( TRUE ); }

       /* This actually generates the tool path */
       UF_CALL( UF_PARAM_generate( tag, &gened ) );
       if( gened == FALSE )
       {
         UF_CALL( UF_UI_open_listing_window() );

         name[0]='\0';
         UF_CALL( UF_OBJ_ask_name(tag, name) );

         str[0]='\0';
         sprintf(str,"\n%s - Generation did not return an error but returned generation FALSE.",name);
         UF_UI_write_listing_window(str);
         return( TRUE );
       }
     }

   }

  return( TRUE );
}

int check_mag1( ) ;
int set_mag1( ) ;
int _set_value_cb ( int dialog_id ) ;
int _list_change ( int dialog_id ) ;
int _action_list_act_cb ( ) ;
int output_str ( int , char *) ;

int ufusr_main( );
int _construct_cb ( int dialog_id ) ;
int _toolpalette_cb ( UF_STYLER_item_value_type_p_t callback_data ) ;
int _list_act_cb ( int dialog_id , UF_STYLER_item_value_type_p_t callback_data ) ;
int _list_dbl_cb ( int dialog_id , UF_STYLER_item_value_type_p_t callback_data ) ;
int _int_tools_cb ( int dialog_id, UF_STYLER_item_value_type_p_t callback_data ) ;
int _blo_tools_act_cb ( int dialog_id, UF_STYLER_item_value_type_p_t callback_data ) ;
int _apply_cb ( int dialog_id ) ;


//----------------------------------------------------------------------------
//  Activation Methods
//----------------------------------------------------------------------------

//  Explicit Activation
//      This entry point is used to activate the application explicitly, as in
//      "File->Execute UG/Open->User Function..."
extern "C" DllExport void ufusr( char *parm, int *returnCode, int rlen )
{
    /* Initialize the API environment */
    int errorCode = UF_initialize();

    if ( 0 == errorCode )
    {
        /* TODO: Add your application code here */
        ufusr_main( );

        /* Terminate the API environment */
        errorCode = UF_terminate();
    }

    /* Print out any error messages */
    PrintErrorMessage( errorCode );
    *returnCode = 0 ;
}

//----------------------------------------------------------------------------
//  Utilities
//----------------------------------------------------------------------------

// Unload Handler
//     This function specifies when to unload your application from Unigraphics.
//     If your application registers a callback (from a MenuScript item or a
//     User Defined Object for example), this function MUST return
//     "UF_UNLOAD_UG_TERMINATE".
extern "C" int ufusr_ask_unload( void )
{
     /* unload immediately after application exits*/
    return ( UF_UNLOAD_IMMEDIATELY );

     /*via the unload selection dialog... */
     //return ( UF_UNLOAD_SEL_DIALOG );
     /*when UG terminates...              */
     //return ( UF_UNLOAD_UG_TERMINATE );
}

/*--------------------------------------------------------------------------
You have the option of coding the cleanup routine to perform any housekeeping
chores that may need to be performed.  If you code the cleanup routine, it is
automatically called by Unigraphics.
--------------------------------------------------------------------------*/
extern void ufusr_cleanup (void)
{
    return;
}


/* PrintErrorMessage
**
**     Prints error messages to standard error and the Unigraphics status
**     line. */
static void PrintErrorMessage( int errorCode )
{
    if ( 0 != errorCode )
    {
        /* Retrieve the associated error message */
        char message[133];
        UF_get_fail_message( errorCode, message );

        /* Print out the message */
        UF_UI_set_status( message );

        fprintf( stderr, "%s\n", message );
    }
}



#define CNT_TOOLS 101

/*
0 NONE  type=-858993460 subtype=-858993460
-----------------------------
1 MILL5    type=1 subtype=1
2 MILL_7_PARAMETER    type=1 subtype=2
3 MILL_10_PARAMETER    type=1 subtype=3
4 BALL_MILL   type=1 subtype=4
-----------------------------
5 DRILL  type=2 subtype=0
-----------------------------
6 BARREL  type=7 subtype=0
-----------------------------
7 T_CUTTER  type=8 subtype=0
-----------------------------
*/
struct tl_i {
  tag_t   tls ; //  

  int tool_type;  // = UF_CUTTER_TYPE_MILL UF_CUTTER_TYPE_DRILL UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  int tool_subtype; // = UF_CUTTER_SUBTYPE_MILL_7 UF_CUTTER_SUBTYPE_MILL_10
  char tool_name[30+1]; //  

  double tool_diameter ; // UF_CUTTER_TYPE_MILL UF_CUTTER_TYPE_DRILL UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  double tool_length ;  // UF_CUTTER_TYPE_MILL UF_CUTTER_TYPE_DRILL UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  double tool_flute_length ;  // UF_CUTTER_TYPE_MILL UF_CUTTER_TYPE_DRILL UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  double tool_taper_angle ; // UF_CUTTER_TYPE_MILL
  double tool_tip_angle ; // UF_CUTTER_TYPE_MILL UF_CUTTER_TYPE_GROOVE
  double tool_corner1_radius ; // UF_CUTTER_TYPE_MILL UF_CUTTER_TYPE_DRILL
  double tool_corner1_center_x ; // UF_CUTTER_SUBTYPE_MILL_7 UF_CUTTER_SUBTYPE_MILL_10
  double tool_corner1_center_y ; // UF_CUTTER_SUBTYPE_MILL_7 UF_CUTTER_SUBTYPE_MILL_10
  double tool_corner2_radius ; // UF_CUTTER_SUBTYPE_MILL_10
  double tool_corner2_center_x ; // UF_CUTTER_SUBTYPE_MILL_10
  double tool_corner2_center_y ; // UF_CUTTER_SUBTYPE_MILL_10
  double tool_barrel_radius ; // UF_CUTTER_TYPE_BARREL
  double tool_shank_diameter ; // UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  double tool_lower_corner_radius ; // UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  double tool_upper_corner_radius ; // UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  double tool_barrel_center_y ; // UF_CUTTER_TYPE_BARREL
  double tool_point_angle ; // UF_CUTTER_TYPE_DRILL
  int tool_flutes_number ; // UF_CUTTER_TYPE_MILL UF_CUTTER_TYPE_DRILL UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  double tool_holder_width ; //
  double tool_max_depth ;
  double tool_z_offset ; // UF_CUTTER_TYPE_MILL UF_CUTTER_TYPE_DRILL UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  double tool_x_offset ;
  double tool_y_offset ;
  int tool_number ; // UF_CUTTER_TYPE_MILL UF_CUTTER_TYPE_DRILL UF_CUTTER_TYPE_BARREL UF_CUTTER_TYPE_T
  int tool_length_adjust_register ;
  int tool_cutcom_register ;
  int tool_direction ;
  char tool_catalog_number[133]; // UF_PARAM_TL_CTLG_NUM
  int tool_insert_position ; // 0   Topside, spindle turns clockwise. 1   Underside, spindle turns counterclockwise.
  int tool_max_depth_toggle ; // 0 || 1
  int tool_adjust_reg_toggle ; // 0 || 1
  int tool_cutcom_reg_toggle ; // 0 || 1
  char tool_cutter_description[133]; // UF_PARAM_TL_DESCRIPTION
  /*     */
}  uTool[CNT_TOOLS];
int uToolCount = 0; //   


/*======================================================================*/
int Tools_read_from_ugs_i(void)
{
    int i;

    tag_t setup_tag, tool_grp;
    tag_t tls;
    tag_t *tool_list = NULL;

    char str[MAX_LINE_SIZE+1] = "";
    char tool_name[UF_PARAM_MAX_STRING_LEN+1];
    char tool_param[UF_PARAM_MAX_STRING_LEN+1];
    int cut_type, cut_subtype;
    int count_t ; // tool
    int status = 0;

    UF_CALL( UF_SETUP_ask_setup(&setup_tag) );
    UF_CALL( UF_SETUP_ask_mct_root(setup_tag, &tool_grp) );
    UF_CALL( UF_NCGROUP_ask_member_list(tool_grp, &count_t, &tool_list) );

    uToolCount=count_t;
    printf ("\n Tools_read_from_ugs_i -> uToolCount=%d",uToolCount);

    //if (count_t==1) {  UF_free(tool_list);   return(count_t); }

    //    0 (    NONE)
    for (i=0;i<count_t;i++)
    {
      if (count_t>=CNT_TOOLS) break ;

      tls=tool_list[i];

      uTool[i].tls=tls;

      UF_CUTTER_ask_type_and_subtype(tls,&cut_type,&cut_subtype);

      uTool[i].tool_type=cut_type;
      uTool[i].tool_subtype=cut_subtype;

      UF_OBJ_ask_name(tls, tool_name);

      uTool[i].tool_name[0]='\0';
      sprintf(uTool[i].tool_name,"%s",tool_name);

      UF_PARAM_ask_double_value(tls,UF_PARAM_TL_DIAMETER,&uTool[i].tool_diameter);
      UF_PARAM_ask_double_value(tls,UF_PARAM_TL_HEIGHT,&uTool[i].tool_length);
      UF_PARAM_ask_double_value(tls,UF_PARAM_TL_FLUTE_LN,&uTool[i].tool_flute_length);
      UF_PARAM_ask_int_value(tls,UF_PARAM_TL_NUM_FLUTES,&uTool[i].tool_flutes_number);

      UF_PARAM_ask_double_value(tls,UF_PARAM_TL_Z_OFFSET,&uTool[i].tool_z_offset);

      UF_PARAM_ask_int_value(tls,UF_PARAM_TL_NUMBER,&uTool[i].tool_number);

      UF_PARAM_ask_str_value(tls,UF_PARAM_TL_CTLG_NUM,tool_param);
      uTool[i].tool_catalog_number[0]='\0';
      if (strlen(tool_param)>0) sprintf(uTool[i].tool_catalog_number,"%s",tool_param);

      UF_PARAM_ask_str_value(tls,UF_PARAM_TL_DESCRIPTION,tool_param);
      uTool[i].tool_cutter_description[0]='\0';
      if (strlen(tool_param)>0) sprintf(uTool[i].tool_cutter_description,"%s",tool_param);

      switch (cut_type) {
        case UF_CUTTER_TYPE_MILL:
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_TAPER_ANG,&uTool[i].tool_taper_angle);
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_TIP_ANG,&uTool[i].tool_tip_angle);
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_COR1_RAD,&uTool[i].tool_corner1_radius);
          if (cut_subtype==UF_CUTTER_SUBTYPE_MILL_7) {
          }
          if (cut_subtype==UF_CUTTER_SUBTYPE_MILL_10) {
            UF_PARAM_ask_double_value(tls,UF_PARAM_TL_COR2_RAD,&uTool[i].tool_corner2_radius);
          }
        break;
        case UF_CUTTER_TYPE_DRILL :
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_COR1_RAD,&uTool[i].tool_corner1_radius);
        break;
        case UF_CUTTER_TYPE_BARREL :
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_BARREL_RAD,&uTool[i].tool_barrel_radius);
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_SHANK_DIA,&uTool[i].tool_shank_diameter);
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_LOW_COR_RAD,&uTool[i].tool_lower_corner_radius);
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_UP_COR_RAD,&uTool[i].tool_upper_corner_radius);
        break;
        case UF_CUTTER_TYPE_T :
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_SHANK_DIA,&uTool[i].tool_shank_diameter);
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_LOW_COR_RAD,&uTool[i].tool_lower_corner_radius);
          UF_PARAM_ask_double_value(tls,UF_PARAM_TL_UP_COR_RAD,&uTool[i].tool_upper_corner_radius);
        break;
        default : break ;
      }

    }

   UF_free(tool_list);

   return (count_t);

}


int output_str ( int i , char *pstr)
{
 char str[MAX_LINE_SIZE+1];
 char tptools[10 + 1];
 int  cut_type , cut_subtype ;

 cut_type = uTool[i].tool_type;
 cut_subtype = uTool[i].tool_subtype;

 tptools[0]='\0';
 switch (cut_type) {
    case UF_CUTTER_TYPE_MILL:
      sprintf(tptools,"%s","mill5");
      if (cut_subtype==UF_CUTTER_SUBTYPE_MILL_7) {
      	sprintf(tptools,"%s","mill7");
      }
      if (cut_subtype==UF_CUTTER_SUBTYPE_MILL_10) {
      	sprintf(tptools,"%s","mill10");
      }
    break;
    case UF_CUTTER_TYPE_DRILL :
       sprintf(tptools,"%s","DRILL");
    break;
    case UF_CUTTER_TYPE_BARREL :
       sprintf(tptools,"%s","BARREL");
    break;
    case UF_CUTTER_TYPE_T :
       sprintf(tptools,"%s","T_CUTTER");
    break;
    default : break ;
 }
 str[0]='\0' ; sprintf(str,"%-30.30s  %-10s  % 10d",uTool[i].tool_name,tptools,uTool[i].tool_number);
 strcpy(pstr, str);

 return(0);
}


int _action_list_act_cb ( )
{
 char str[MAX_LINE_SIZE+1];
 int  i ;

 UF_UI_open_listing_window();

 //  
 for (i=0;i<uToolCount;i++)
 {
   if (0==strcmp(uTool[i].tool_name,"NONE")) continue ;

   str[0]='\0';
   sprintf(str,"\n % 2d) ",i);
   UF_UI_write_listing_window(str);

   str[0]='\0';
   output_str ( i , str) ;

   //UF_UI_write_listing_window("\n");
   UF_UI_write_listing_window(str);

   str[0]='\0';
   sprintf(str,"   D=%.3f  r1=%.3f  FL=%.3f  L=%.3f",uTool[i].tool_diameter,uTool[i].tool_corner1_radius,uTool[i].tool_flute_length,uTool[i].tool_length);
   UF_UI_write_listing_window(str);

 }

 return(0);
}

/*======================================================================*/
int check_mag1( )
{
  UF_UI_open_listing_window();

UF_UI_write_listing_window("\n   MAG3");
UF_UI_write_listing_window("\n---------------------------------------");
UF_UI_write_listing_window("\n                             ");
UF_UI_write_listing_window("\n---------------------------------------");
UF_UI_write_listing_window("\n  1  .  D50R3             50 ");
UF_UI_write_listing_window("\n  2  D25R3                         26 ");
UF_UI_write_listing_window("\n  3  D20R3 BAR3500R202SA20SB       22 ");
UF_UI_write_listing_window("\n  4  D20R3 ()               23 ");
UF_UI_write_listing_window("\n  5  D20R0                         20 ");
UF_UI_write_listing_window("\n  6  D16R3                         17 ");
UF_UI_write_listing_window("\n  7  D16R0 (Lp=25)                 15 ");
UF_UI_write_listing_window("\n  8  D16R0 (Lp=50)                 16 ");
UF_UI_write_listing_window("\n  9  D12R3 (Lp=35)                 14 ");
UF_UI_write_listing_window("\n 10  D12R3 (Lp=70)                 13 ");
UF_UI_write_listing_window("\n 11  D12R0                         12 ");
UF_UI_write_listing_window("\n 12   D12                    25 ");
UF_UI_write_listing_window("\n 13  D10R3                         11 ");
UF_UI_write_listing_window("\n 14  D10R0                         10 ");
UF_UI_write_listing_window("\n 15  D8R0                          8  ");
UF_UI_write_listing_window("\n 16   D5                     5  ");
UF_UI_write_listing_window("\n 17   D4                     4  ");
UF_UI_write_listing_window("\n 18   D3.1                   3  ");
UF_UI_write_listing_window("\n 19   D2.7                   2  ");
UF_UI_write_listing_window("\n 20    D2        1  ");
UF_UI_write_listing_window("\n 21  .  D40R3H15            43 ");
UF_UI_write_listing_window("\n 22  .  D30R3H12            44 ");
UF_UI_write_listing_window("\n");
UF_UI_write_listing_window("\n---------------------------------------");

  return(0);
}


/*======================================================================*/
int set_mag1( )
{
 int  cut_type , cut_subtype ;
 int T; double D,L,FL,r1;
 int i , cnt = 0 ;

 //  
 for (i=0,cnt = 0;i<uToolCount;i++)
 {
   if (0==strcmp(uTool[i].tool_name,"NONE")) continue ;

   T = uTool[i].tool_number ;
   D = uTool[i].tool_diameter;
   L = uTool[i].tool_length;
   FL =uTool[i].tool_flute_length;
   r1 =uTool[i].tool_corner1_radius;

   cut_type = uTool[i].tool_type;
   cut_subtype = uTool[i].tool_subtype;

   switch (cut_type) {
    case UF_CUTTER_TYPE_MILL:

      if (D==50 && r1==3) T= 50;
      if (D==25 && r1==3) T= 26;

      if (D==20 && r1==3 && (0==strcmp(uTool[i].tool_name,"BAR3500")) ) T= 22;
      if (D==20 && r1==3) T= 23;

      if (D==20 && r1==0) T= 20;

      if (D==16 && r1==3) T= 17;

      if (D==16 && r1==0 && L==25) T= 15;
      if (D==16 && r1==0 && L==50) T= 16;

      if (D==12 && r1==3 && L==35) T= 14;
      if (D==12 && r1==3 && L==70) T= 13;

      if (D==12 && r1==0) T= 12;
      if (D==10 && r1==3) T= 11;
      if (D==10 && r1==0) T= 10;
      if (D== 8 && r1==0) T=  8;

      if (cut_subtype==UF_CUTTER_SUBTYPE_MILL_7) {
      }
      if (cut_subtype==UF_CUTTER_SUBTYPE_MILL_10) {
      }
    break;
    case UF_CUTTER_TYPE_DRILL :
       if (D==12 ) T= 25;
       if (D==5  ) T=  5;
       if (D==4  ) T=  4;
       if (D==3.1) T=  3;
       if (D==2.7) T=  2;
       if (D==2  ) T=  1;
    break;
    case UF_CUTTER_TYPE_BARREL :
    break;
    case UF_CUTTER_TYPE_T :
       if (D==40 && r1==3 && FL==15) T= 43;
       if (D==30 && r1==3 && FL==12) T= 44;
    break;
    default : break ;
   }

   if (T!=uTool[i].tool_number ) { uTool[i].tool_number = T ; cnt++ ; }

 }

  return(cnt);
}


/*********************************************



*********************************************/
int ufusr_main( )
{
  int errorCode;
  int response ;

  int module_id;
  UF_ask_application_module(&module_id);
  if (UF_APP_CAM!=module_id) {
    uc1601(" DLL -    CAM \n, 2007.\nDLL terminated...",1);
    return(0) ;
  }

  if (NULL_TAG==UF_PART_ask_display_part()) {
      uc1601("Cam-  .....\n  .",1);
      return (0);
  }

  /****************************************************************************/

    if ( ( errorCode = UF_STYLER_create_dialog (
            actions->styler_file,
           MT_cbs,      /* Callbacks from dialog */
           MT_CB_COUNT, /* number of callbacks*/
            NULL,        /* This is your client data */
            &response ) ) != 0 )
    {
       char fail_message[133];

       /* Get the user function fail message based on the fail code.*/
       UF_get_fail_message(errorCode, fail_message);
       UF_UI_set_status (fail_message);
       printf ( "%s\n", fail_message );

       uc1601("   \n    dlg-.",1);
       return (-1);
    }

  /****************************************************************************/

  return(0);
}


int _construct_cb ( int dialog_id )
{
 int jj = Tools_read_from_ugs_i( ) ; //  
 _set_value_cb ( dialog_id ) ;
 return(0);
}


int _list_change ( int dialog_id )
{
 char str[MAX_LINE_SIZE+1];
 int  val , i , ind ;
 UF_STYLER_item_value_type_t data  ;

 data.item_attr=UF_STYLER_VALUE;

 data.item_id=MT_INT_TOOLS;
 UF_STYLER_ask_value(dialog_id,&data);
 val = data.value.integer ;

 //  
 data.item_id=MT_LIST_TOOLS;
 data.indicator=UF_STYLER_INTEGER_VALUE; // UF_STYLER_INTEGER_VALUE UF_STYLER_STRING_VALUE
 UF_STYLER_ask_value(dialog_id,&data);
 ind=data.subitem_index; // str[0]='\0'; sprintf(str,"%s",data.value.string);
 if (ind<0) ind=data.value.integer;
 //  
 data.item_attr=UF_STYLER_LIST_DELETE ;
 data.subitem_index=ind ; // UF_STYLER_NO_SUB_INDEX
 UF_STYLER_set_value(dialog_id,&data);
 //   
 data.item_attr=UF_STYLER_LIST_INSERT ;
 data.subitem_index=ind ;
 data.indicator = UF_STYLER_STRING_VALUE ;

   i=ind+1 ;
   uTool[i].tool_number = val ;
   str[0]='\0';
   output_str ( i , str) ;

 data.value.string = str;
 UF_STYLER_set_value(dialog_id,&data);

 data.item_attr=UF_STYLER_VALUE;
 UF_STYLER_set_value(dialog_id,&data);

 UF_STYLER_free_value (&data) ;
 return(0);
}

int _toolpalette_cb ( UF_STYLER_item_value_type_p_t callback_data )
{
 int ind=callback_data->value.integer;
 if (ind==0) UF_CALL( UF_UI_ONT_switch_view( UF_UI_ONT_order ) );
 if (ind==1) UF_CALL( UF_UI_ONT_switch_view( UF_UI_ONT_machine_tool ) );
 return(0);
}


int _list_act_cb ( int dialog_id , UF_STYLER_item_value_type_p_t callback_data )
{
 UF_STYLER_item_value_type_t data  ;
 int ind = callback_data->value.integer ;
 if (ind < 0 ) return (0);
 ind++ ;  if (ind>=CNT_TOOLS) { ind=CNT_TOOLS ; }

 data.item_id=MT_INT_TOOLS;
 data.item_attr=UF_STYLER_VALUE;
 data.value.integer= uTool[ind].tool_number ;
 UF_STYLER_set_value(dialog_id,&data);

 UF_STYLER_free_value (&data) ;

 return(0);
}


int _list_dbl_cb ( int dialog_id , UF_STYLER_item_value_type_p_t callback_data )
{
 int  response , ia4[ 1 ], ip5 ;
 char cp1[80] ,  cp2[ 1 ][ 16 ] ;

 int ind = callback_data->value.integer ;
 if (ind < 0 ) return (0);
 ind++ ;  if (ind>=CNT_TOOLS) { ind=CNT_TOOLS ; }

 /********************************************************************************/
 cp1[0]='\0' ; sprintf (cp1,"%s", uTool[ind].tool_name);
 strcpy(&cp2[0][0], "T=");
 ia4[ 0 ] = uTool[ind].tool_number ;

 response = uc1607 (cp1, cp2, 1, ia4, &ip5 );
 //if (response != 3 && response != 4) { return (-1); }
 if (response != 4) { return (-1); }
 /********************************************************************************/

 if (ia4[ 0 ]<0) { return (-2); }
 if (uTool[ind].tool_number == ia4[ 0 ]) { return (-2); }

 UF_STYLER_item_value_type_t data  ;

 data.item_id=MT_INT_TOOLS;
 data.item_attr=UF_STYLER_VALUE;
 data.value.integer= ia4[ 0 ] ;
 UF_STYLER_set_value(dialog_id,&data);

 _list_change ( dialog_id ) ;

 UF_STYLER_free_value (&data) ;
 return(0);
}


int _int_tools_cb ( int dialog_id, UF_STYLER_item_value_type_p_t callback_data )
{
 UF_STYLER_item_value_type_t data  ;
 if (callback_data->value.integer<0) {
   UF_UI_set_status ("Number T must > 0 ; fixed");
 	 callback_data->value.integer = 0;

   data.item_id=MT_INT_TOOLS;
   data.item_attr=UF_STYLER_VALUE;
   data.value.integer= callback_data->value.integer ;
   UF_STYLER_set_value(dialog_id,&data);
 }
 _list_change ( dialog_id ) ;
 UF_STYLER_free_value (&data) ;
 return(0);
}


int _blo_tools_act_cb ( int dialog_id, UF_STYLER_item_value_type_p_t callback_data)
{
  int i ;
  char str[MAX_LINE_SIZE+1];
  int ind = callback_data->value.integer;

    if (ind==0) // check Mag3
    {
    	check_mag1();
    }
    if (ind==1) // set Mag3
    {
    	i=set_mag1();
    	if (i<=0) {
       str[0]='\0'; sprintf (str, "   = %d\n    :  - .", i );
       uc1601(str,TRUE);    	
    	} else { 
    	 _set_value_cb ( dialog_id ) ;
       str[0]='\0'; sprintf (str, "  = %d\n    :  - .", i );
       uc1601(str,TRUE);
    	}
    }

    return (0);
}


int _set_value_cb ( int dialog_id )
{
 UF_STYLER_item_value_type_t data  ;
 int i , j;
 char str[MAX_LINE_SIZE+1];

 data.item_id=MT_LIST_TOOLS;

// 
 data.item_attr=UF_STYLER_LIST_DELETE;
 data.subitem_index=UF_STYLER_NO_SUB_INDEX;
 UF_STYLER_set_value(dialog_id,&data);

 data.item_attr=UF_STYLER_LIST_INSERT;
 data.indicator=UF_STYLER_STRING_VALUE;

 //    0 ( NONE)
 for (i=0+0,j=0;i<uToolCount;i++)
 {
   if (0==strcmp(uTool[i].tool_name,"NONE")) {j--;continue ;}

   str[0]='\0';
   output_str ( i , str) ;

   data.subitem_index=i;
   data.value.string=str;
   UF_STYLER_set_value(dialog_id,&data);
 }

 UF_STYLER_free_value (&data) ;

 return(0);
}




int _apply_cb ( int dialog_id )
{
  /*  Variable Declarations */
  char   str[133];
  int    ecode ;
  int    all_oper ;
  int    num ;
  tag_t  tls = NULL_TAG;
  int    i , count = 0 ;

  int  response ;
  char *mes2[]={
      "      ?",
      NULL
  };
  UF_UI_message_buttons_t buttons2 = { TRUE, FALSE, TRUE, "..", NULL, "", 1, 0, 2 };

 /********************************************************************************/
 UF_STYLER_item_value_type_t data  ;
 data.item_id=MT_TOGGLE_TOOLS;
 data.item_attr=UF_STYLER_VALUE;
 UF_STYLER_ask_value(dialog_id,&data);
 all_oper = data.value.integer ;
 UF_STYLER_free_value (&data) ;
 /********************************************************************************/
 if (all_oper!=0) {
   UF_UI_message_dialog("Cam.....", UF_UI_MESSAGE_QUESTION, mes2, 1, TRUE, &buttons2, &response);
   if (response==2) { response=0; }
 }
 /********************************************************************************/

 UF_UI_toggle_stoplight(1);

 //    1 (    NONE)
 for (i=0,count = 0;i<uToolCount;i++)
 {
   if (0==strcmp(uTool[i].tool_name,"NONE")) continue ;
   tls=uTool[i].tls;
   ecode = UF_CALL( UF_PARAM_ask_int_value(tls,UF_PARAM_TL_NUMBER,&num) );
   if (uTool[i].tool_number!=num) {
     ecode = UF_CALL( UF_PARAM_set_int_value(tls,UF_PARAM_TL_NUMBER,-99) );
     ecode = UF_CALL( UF_PARAM_set_int_value(tls,UF_PARAM_TL_NUMBER,uTool[i].tool_number) );
     count ++ ;
     if (all_oper!=0)
     {
      num_tool = uTool[i].tool_number ;
      generat  = response ;
     	UF_CALL( UF_NCGROUP_cycle_members( tls, cycleGeneratePrg,NULL ) );
     }
   }
  /*
   UF_PARAM_ask_str_value(tls,UF_PARAM_TL_CTLG_NUM,tool_param);
   uTool[i].tool_catalog_number[0]='\0';
   if (strlen(tool_param)>0) sprintf(uTool[i].tool_catalog_number,"%s",tool_param);
   UF_PARAM_ask_str_value(tls,UF_PARAM_TL_DESCRIPTION,tool_param);
   uTool[i].tool_cutter_description[0]='\0';
   if (strlen(tool_param)>0) sprintf(uTool[i].tool_cutter_description,"%s",tool_param);
  */
 }

 //UF_DISP_refresh ();
 UF_UI_ONT_refresh();

 UF_UI_toggle_stoplight(0);

 str[0]='\0'; sprintf(str,"  = %d (  %d ).",count,uToolCount-1);
 uc1601(str,1);

 return (0);
}




/*-------------------------------------------------------------------------*/
/*---------------------- UIStyler Callback Functions ----------------------*/
/*-------------------------------------------------------------------------*/

/* -------------------------------------------------------------------------
 * Callback Name: MT_construct_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the NX Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int MT_construct_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _construct_cb ( dialog_id ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must continue dialog construction.*/

}


/* -------------------------------------------------------------------------
 * Callback Name: MT_apply_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the NX Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int MT_apply_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _apply_cb ( dialog_id ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog                 */
    /* A return value of UF_UI_CB_EXIT_DIALOG will not be accepted    */
    /* for this callback type.  You must respond to your apply button.*/
    return (UF_UI_CB_CONTINUE_DIALOG);

}


/* -------------------------------------------------------------------------
 * Callback Name: MT_toolpalette_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the NX Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int MT_toolpalette_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _toolpalette_cb ( callback_data ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: MT_list_act_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the NX Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int MT_list_act_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _list_act_cb ( dialog_id , callback_data ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: MT_list_dbl_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the NX Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int MT_list_dbl_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _list_dbl_cb ( dialog_id , callback_data ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: MT_int_tools_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the NX Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int MT_int_tools_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _int_tools_cb ( dialog_id, callback_data ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: MT_action_list_act_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the NX Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int MT_action_list_act_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _action_list_act_cb ( ) ;

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}


/* -------------------------------------------------------------------------
 * Callback Name: MT_tog_tools_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the NX Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int MT_tog_tools_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.  */
    /* return ( UF_UI_CB_EXIT_DIALOG );             */

}


/* -------------------------------------------------------------------------
 * Callback Name: MT_blo_tools_act_cb
 * This is a callback function associated with an action taken from a
 * UIStyler object.
 *
 * Input: dialog_id   -   The dialog id indicate which dialog this callback
 *                        is associated with.  The dialog id is a dynamic,
 *                        unique id and should not be stored.  It is
 *                        strictly for the use in the NX Open API:
 *                               UF_STYLER_ask_value(s)
 *                               UF_STYLER_set_value
 *        client_data -   Client data is user defined data associated
 *                        with your dialog.  Client data may be bound
 *                        to your dialog with UF_MB_add_styler_actions
 *                        or UF_STYLER_create_dialog.
 *        callback_data - This structure pointer contains information
 *                        specific to the UIStyler Object type that
 *                        invoked this callback and the callback type.
 * -----------------------------------------------------------------------*/
int MT_blo_tools_act_cb ( int dialog_id,
             void * client_data,
             UF_STYLER_item_value_type_p_t callback_data)
{
     /* Make sure User Function is available. */
     if ( UF_initialize() != 0)
          return ( UF_UI_CB_CONTINUE_DIALOG );

     /* ---- Enter your callback code here ----- */
     _blo_tools_act_cb ( dialog_id, callback_data );

     UF_terminate ();

    /* Callback acknowledged, do not terminate dialog */
    return (UF_UI_CB_CONTINUE_DIALOG);

    /* or Callback acknowledged, terminate dialog.    */
    /* return ( UF_UI_CB_EXIT_DIALOG );               */

}



